DeviceString

Struct DeviceString 

pub struct DeviceString(/* private fields */);
String limited to 30 UTF-16 code units.

The strings stored in the MCP2221 flash memory (used during USB enumeration) are limited to at most 60 bytes of UTF-16-encoded text.

Create a DeviceString by calling str::parse on a string slice, or DeviceString::try_from with an owned String.

let manufacturer: DeviceString = "Acme Widgets Company (UK) Ltd".parse()?;

let product = String::from("Internet of Widgets Hub v3.0");
let product: DeviceString = product.try_into()?;

Note that some characters require two UTF-16 code units to express (4 bytes).

let serial = "4 bytes each: ๐Ÿซ๐Ÿซ‘๐Ÿซ’๐Ÿซ“๐Ÿซ”๐Ÿซ•๐Ÿซ–๐Ÿซ—๐Ÿซ˜๐Ÿซ™";
let result: Result<DeviceString, _> = serial.parse();
assert!(result.is_err(), "More than 60 bytes when UTF-16 encoded.");

Datasheet

See table 3-7 and table 3-14 for details of how the device strings are read from and written to the MCP2221, including the length limitation. (Those tables are for the manufacturer string, but the following tables are identical except for the subcommand code.)